import Button from '@material-ui/core/Button'
import CloseIcon from 'assets/icons/ico/close.svg'
import StyledIconButton from 'components/CommonKit/IconButton/StyledIconButton'
import QuizExplanationModal from 'components/Quiz/QuizExplanationModal/QuizExplanationModal'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/providers/I18n'
import { UserChallengeUpdateFlag } from 'enums'
import { QuestionEntity, UserChallenge } from 'models'
import React, { useState } from 'react'
import ChallengeService from 'services/challenge.service'
import QuizService from 'services/quiz.service'
import { updateUserChallengeList } from 'store/challenge/challenge.slice'
import { useAppDispatch } from 'store/hooks'
import './quizQuestion.scss'

interface QuizQuestionContentCustomProps {
  userChallenge: UserChallenge
  goBack: () => void
  customQuestion: QuestionEntity
}

const QuizQuestionContentCustom = ({
  userChallenge,
  goBack,
  customQuestion,
}: QuizQuestionContentCustomProps) => {
  const { t } = useI18n()
  const client = useClient()
  const dispatch = useAppDispatch()

  const [userChoice, setUserChoice] = useState<string>('')
  const [openModal, setOpenModal] = useState<boolean>(false)
  const [answerIndex, setAnswerIndex] = useState<number>(0)
  const quizService = new QuizService(client)
  const challengeService = new ChallengeService(client)

  const validateQuestion = async () => {
    const resultIndex = customQuestion.answers.findIndex(
      answer => answer.answerLabel === userChoice
    )
    setAnswerIndex(resultIndex)
    setOpenModal(true)

    const result = customQuestion.answers.filter(
      answer => answer.answerLabel === userChoice
    )
    const quizUpdated = quizService.updateUserQuiz(
      userChallenge.quiz,
      result[0].isTrue
    )
    const userChallengeUpdated = await challengeService.updateUserChallenge(
      userChallenge,
      UserChallengeUpdateFlag.QUIZ_UPDATE,
      quizUpdated
    )
    dispatch(updateUserChallengeList(userChallengeUpdated))
  }

  const handleChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setUserChoice(e.target.value)
  }

  const finishQuiz = async () => {
    setOpenModal(false)
    const userChallengeUpdated = await challengeService.updateUserChallenge(
      userChallenge,
      UserChallengeUpdateFlag.QUIZ_DONE
    )
    dispatch(updateUserChallengeList(userChallengeUpdated))
  }

  return (
    <div className="quiz-container">
      <div className="question-container">
        <StyledIconButton
          className="btn-back"
          icon={CloseIcon}
          onClick={goBack}
        />
        <p className="index-question">5/5</p>
        <p className="question-title text-18-bold">
          {t('quiz.consumption_question')}
        </p>

        <p className="question text-18-bold">{customQuestion.questionLabel}</p>
        {customQuestion.answers.map((answer, index) => (
          <div className="answer" key={index}>
            <input
              type="radio"
              id={`answer${index}`}
              value={answer.answerLabel}
              onChange={handleChange}
              checked={userChoice === answer.answerLabel}
            />
            <label htmlFor={`answer${index}`} className="text-16-bold">
              {answer.answerLabel}
            </label>
          </div>
        ))}
      </div>
      <Button
        aria-label={t('quiz.accessibility.button_validate')}
        onClick={validateQuestion}
        disabled={!userChoice}
        className="btnSecondary validate"
      >
        {t('quiz.button_validate')}
      </Button>

      <QuizExplanationModal
        open={openModal}
        answerIndex={answerIndex}
        question={customQuestion}
        goNext={finishQuiz}
        handleCloseClick={() => setOpenModal(false)}
      />
    </div>
  )
}

export default QuizQuestionContentCustom
