import { Season, UserActionState } from 'enums'
import { DateTime } from 'luxon'
import { Ecogesture, UserAction } from 'models'
import {
  AllEcogestureData,
  defaultEcogestureData,
  ecogestureDefault,
} from 'tests/__mocks__/actionData.mock'
import mockClient from 'tests/__mocks__/client.mock'
import { userChallengeData } from 'tests/__mocks__/userChallengeData.mock'
import ActionService from './action.service'

const mockGetAllUserChallengeEntities = jest.fn()
jest.mock('./challenge.service', () => {
  return jest.fn(() => ({
    getAllUserChallengeEntities: mockGetAllUserChallengeEntities,
  }))
})

const mockGetAllEcogestures = jest.fn()
const mockGetEcogesturesByIds = jest.fn()
jest.mock('./ecogesture.service', () => {
  return jest.fn(() => ({
    getAllEcogestures: mockGetAllEcogestures,
    getEcogesturesByIds: mockGetEcogesturesByIds,
  }))
})

const mockGetSeason = jest.fn()
jest.mock('utils/utils', () => ({
  getSeason: () => mockGetSeason(),
}))

describe('Action Service', () => {
  const actionService = new ActionService(mockClient)

  it('should return the default actions', async () => {
    mockGetAllEcogestures.mockResolvedValueOnce(AllEcogestureData)
    mockGetEcogesturesByIds.mockResolvedValueOnce(defaultEcogestureData)

    const result: Ecogesture[] = await actionService.getDefaultActions()
    expect(result).toEqual(defaultEcogestureData)
  })
  it('should return the Available Action List', async () => {
    mockGetAllUserChallengeEntities.mockResolvedValueOnce(userChallengeData)
    mockGetAllEcogestures.mockResolvedValueOnce(AllEcogestureData)
    mockGetEcogesturesByIds.mockResolvedValueOnce([
      AllEcogestureData[2],
      AllEcogestureData[3],
      AllEcogestureData[4],
    ])

    const result: Ecogesture[] = await actionService.getAvailableActionList()
    expect(result).toEqual([
      AllEcogestureData[2],
      AllEcogestureData[3],
      AllEcogestureData[4],
    ])
  })
  it('should return winter ecogestures', () => {
    mockGetSeason.mockReturnValueOnce(Season.WINTER)
    const result: Ecogesture[] = actionService.filterBySeason(AllEcogestureData)
    expect(result).toEqual([
      AllEcogestureData[0],
      AllEcogestureData[2],
      AllEcogestureData[5],
    ])
  })
  it('should return no season ecogestures', () => {
    mockGetSeason.mockReturnValueOnce(Season.NONE)
    const result: Ecogesture[] = actionService.filterBySeason(AllEcogestureData)

    expect(result).toEqual([AllEcogestureData[3], AllEcogestureData[4]])
  })
  it('should return summer ecogestures', () => {
    mockGetSeason.mockReturnValueOnce(Season.SUMMER)
    const result: Ecogesture[] = actionService.filterBySeason(AllEcogestureData)

    expect(result).toEqual([
      AllEcogestureData[1],
      AllEcogestureData[3],
      AllEcogestureData[4],
    ])
  })

  it('should launch an action', () => {
    jest
      .spyOn(DateTime, 'local')
      .mockReturnValueOnce(
        DateTime.fromISO('2020-10-01T00:00:00.000Z', { zone: 'utc' })
      )
    const result: UserAction = actionService.launchAction(ecogestureDefault)
    const UserActionDefault: UserAction = {
      ecogesture: ecogestureDefault,
      startDate: DateTime.fromISO('2020-10-01T00:00:00.000Z', {
        zone: 'utc',
      }),
      state: UserActionState.ONGOING,
    }
    expect(result).toEqual(UserActionDefault)
  })
  it('complete list if < 3', async () => {
    mockGetEcogesturesByIds.mockResolvedValueOnce([
      AllEcogestureData[2],
      AllEcogestureData[3],
      AllEcogestureData[4],
    ])
    const result = await actionService.keepListComplete([
      AllEcogestureData[0],
      AllEcogestureData[1],
    ])

    expect(result).toEqual([
      AllEcogestureData[0],
      AllEcogestureData[1],
      AllEcogestureData[2],
    ])
  })

  describe('getCustomActions function', () => {
    it('should return filtered and sorted ecogestures and not complete the list', async () => {
      mockGetSeason.mockReturnValueOnce(Season.WINTER)
      mockGetAllUserChallengeEntities.mockResolvedValueOnce(userChallengeData)
      mockGetAllEcogestures.mockResolvedValueOnce(AllEcogestureData)
      mockGetEcogesturesByIds.mockResolvedValueOnce([
        AllEcogestureData[0],
        AllEcogestureData[1],
        AllEcogestureData[2],
        AllEcogestureData[3],
        AllEcogestureData[4],
        AllEcogestureData[5],
      ])

      const result = await actionService.getCustomActions([0, 2])
      expect(result).toEqual([
        AllEcogestureData[0],
        AllEcogestureData[5],
        AllEcogestureData[2],
      ])
    })
    it('should return filtered and sorted ecogestures and complete the list with default ecogesture', async () => {
      mockGetSeason.mockReturnValueOnce(Season.WINTER)
      mockGetAllUserChallengeEntities.mockResolvedValueOnce(userChallengeData)
      mockGetAllEcogestures.mockResolvedValueOnce(AllEcogestureData)
      mockGetEcogesturesByIds.mockResolvedValueOnce([
        AllEcogestureData[0],
        AllEcogestureData[1],
        AllEcogestureData[2],
        AllEcogestureData[3],
      ])
      mockGetEcogesturesByIds.mockClear()
      mockGetEcogesturesByIds.mockResolvedValueOnce([
        defaultEcogestureData[0],
        defaultEcogestureData[1],
        defaultEcogestureData[2],
      ])
      const result = await actionService.getCustomActions([0, 2])
      expect(result).toEqual([
        AllEcogestureData[0],
        defaultEcogestureData[0],
        AllEcogestureData[2],
      ])
    })
  })
})
